@@ -24,10 +24,6 @@ extern "C" {
  *
  ***********************************************************************/
 
-extern Datum SPI_initialize(PG_FUNCTION_ARGS);
-
-#if (PGSQL_MAJOR_VER >= 8)
-
 typedef struct
 {
 	SubTransactionId xid;
@@ -35,12 +31,19 @@ typedef struct
 	char name[1];
 } Savepoint;
 
+/* infant is set to the savepoint that is being created durin a setSavepoint call.
+ * It is used by the onStart callback.
+ */
+extern Savepoint* infant;
+
 extern Savepoint* SPI_setSavepoint(const char* name);
 
 extern void SPI_releaseSavepoint(Savepoint* sp);
 
 extern void SPI_rollbackSavepoint(Savepoint* sp);
 
+#if (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER < 1)
+extern char* SPI_getnspname(Relation rel);
 #endif
 
 #ifdef __cplusplus
