@@ -9,7 +9,7 @@
 #ifndef __pljava_ErrorData_h
 #define __pljava_ErrorData_h
 
-#include "pljava/type/NativeStruct.h"
+#include "pljava/type/JavaWrapper.h"
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -20,26 +20,16 @@ extern "C" {
  * @author Thomas Hallgren
  *****************************************************************/
 
-#if (PGSQL_MAJOR_VER < 8)
-
-typedef struct ErrorData
-{
-	int   elevel;		/* error level */
-	int   sqlerrcode;	/* encoded ERRSTATE */
-	char* message;		/* primary error message */
-} ErrorData;
-
-#endif
-
 /*
- * Create the org.postgresql.pljava.internal.ErrorData instance
+ * Create the org.postgresql.pljava.internal.ErrorData that represents
+ * the current error obtaind from CopyErrorData().
  */
-extern jobject ErrorData_create(JNIEnv* env, ErrorData* errorData);
+extern jobject ErrorData_getCurrentError(void);
 
 /*
  * Extract the native ErrorData from a Java ErrorData.
  */
-extern ErrorData* ErrorData_getErrorData(JNIEnv* env, jobject jerrorData);
+extern ErrorData* ErrorData_getErrorData(jobject jerrorData);
 
 
 #ifdef __cplusplus
