@@ -9,9 +9,9 @@
 #include "pljava/PgObject_priv.h"
 #include "pljava/backports.h"
 #include "pljava/Exception.h"
-#include "pljava/Backend.h"
+#include "pljava/Invocation.h"
+#include "pljava/Function.h"
 #include "pljava/HashMap.h"
-#include "pljava/MemoryContext.h"
 #include "pljava/type/Oid.h"
 #include "pljava/type/SingleRowWriter.h"
 #include "pljava/type/ResultSetProvider.h"
@@ -93,28 +93,21 @@ static void _Function_finalize(PgObject self)
 	}
 }
 
-PG_FUNCTION_INFO_V1(Function_initialize);
 
 static jclass s_Loader_class;
 static jmethodID s_Loader_getSchemaLoader;
 
-Datum Function_initialize(PG_FUNCTION_ARGS)
+extern void Function_initialize(void);
+void Function_initialize(void)
 {
-	JNIEnv* env = (JNIEnv*)PG_GETARG_POINTER(0);
-
 	s_funcMap = HashMap_create(59, TopMemoryContext);
 	s_FunctionClass = PgObjectClass_create("Function", sizeof(struct Function_), _Function_finalize);
 	
-	s_Loader_class = (*env)->NewGlobalRef(
-						env, PgObject_getJavaClass(env, "org/postgresql/pljava/sqlj/Loader"));
-	s_Loader_getSchemaLoader = PgObject_getStaticJavaMethod(
-						env, s_Loader_class, "getSchemaLoader", "(Ljava/lang/String;)Ljava/lang/ClassLoader;");
+	s_Loader_class = JNI_newGlobalRef(PgObject_getJavaClass("org/postgresql/pljava/sqlj/Loader"));
+	s_Loader_getSchemaLoader = PgObject_getStaticJavaMethod(s_Loader_class, "getSchemaLoader", "(Ljava/lang/String;)Ljava/lang/ClassLoader;");
 
-	s_ClassLoader_class = (*env)->NewGlobalRef(
-						env, PgObject_getJavaClass(env, "java/lang/ClassLoader"));
-	s_ClassLoader_loadClass = PgObject_getJavaMethod(
-						env, s_ClassLoader_class, "loadClass", "(Ljava/lang/String;)Ljava/lang/Class;");
-	PG_RETURN_VOID();
+	s_ClassLoader_class = JNI_newGlobalRef(PgObject_getJavaClass("java/lang/ClassLoader"));
+	s_ClassLoader_loadClass = PgObject_getJavaMethod(s_ClassLoader_class, "loadClass", "(Ljava/lang/String;)Ljava/lang/Class;");
 }
 
 static void Function_buildSignature(Function self, StringInfoData* sign, Type retType)
@@ -215,14 +208,13 @@ static void Function_parseParameters(Function self, Oid* dfltIds, const char* pa
 	}
 }
 
-static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
+static void Function_init(Function self, PG_FUNCTION_ARGS)
 {
 	const char* ip;
 	const char* paramDecl = 0;
 	const char* methodName;
 	char* cp;
 	char* className;
-	bool  saveIcj = isCallingJava;
 	bool  isNull = false;
 	HeapTuple nspTup;
 	Form_pg_namespace nspStruct;
@@ -325,33 +317,25 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 	nspStruct = (Form_pg_namespace)GETSTRUCT(nspTup);
 
 	ip = NameStr(nspStruct->nspname);
-	schemaName = String_createJavaStringFromNTS(env, ip);
+	schemaName = String_createJavaStringFromNTS(ip);
 
 	elog(DEBUG1, "Obtaining classloader for schema %s", ip);
-	isCallingJava = true;
-	loader = (*env)->CallStaticObjectMethod(env, s_Loader_class, s_Loader_getSchemaLoader, schemaName);
-	isCallingJava = saveIcj;
-
-	(*env)->DeleteLocalRef(env, schemaName);
+	loader = JNI_callStaticObjectMethod(s_Loader_class, s_Loader_getSchemaLoader, schemaName);
+	JNI_deleteLocalRef(schemaName);
 	ReleaseSysCache(nspTup);
-	Exception_checkException(env);
 
-	jname  = String_createJavaStringFromNTS(env, className);
+	jname  = String_createJavaStringFromNTS(className);
 
 	elog(DEBUG1, "Loading class %s", className);
-	isCallingJava = true;
-	loaded = (*env)->CallObjectMethod(env, loader, s_ClassLoader_loadClass, jname);
-	isCallingJava = saveIcj;
+	loaded = JNI_callObjectMethod(loader, s_ClassLoader_loadClass, jname);
 
-	(*env)->DeleteLocalRef(env, jname);
-	(*env)->DeleteLocalRef(env, loader);
-
-	Exception_checkException(env);
+	JNI_deleteLocalRef(jname);
+	JNI_deleteLocalRef(loader);
 
 	self->returnComplex = false;
 	self->readOnly = (procStruct->provolatile != PROVOLATILE_VOLATILE);
-	self->clazz = (jclass)(*env)->NewGlobalRef(env, loaded);
-	(*env)->DeleteLocalRef(env, loaded);
+	self->clazz = (jclass)JNI_newGlobalRef(loaded);
+	JNI_deleteLocalRef(loaded);
 
 	methodName = cp;
 
@@ -440,7 +424,7 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 		if(top > 0)
 		{
 			int idx;
-#if (PGSQL_MAJOR_VER < 8 || (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER == 0))
+#if (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER == 0)
 			Oid* typeIds = procStruct->proargtypes;
 #else
 			Oid* typeIds = procStruct->proargtypes.values;
@@ -459,12 +443,7 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 				{
 					self->paramTypes[idx] = Type_fromJavaType(
 						InvalidOid,
-#if (PGSQL_MAJOR_VER < 8)
-						"org.postgresql.pljava.jdbc.SingleRowReader"
-#else
-						"org.postgresql.pljava.jdbc.SingleTupleReader"
-#endif
-						);
+						"org.postgresql.pljava.jdbc.SingleTupleReader");
 				}
 				else
 					self->paramTypes[idx] = Type_fromPgType(typeId, pgType);
@@ -490,10 +469,7 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 	ReleaseSysCache(procTup);
 
 	elog(DEBUG1, "Obtaining method %s.%s %s", className, methodName, sign.data);
-	isCallingJava = true;
-	self->method = (*env)->GetStaticMethodID(
-						env, self->clazz, methodName, sign.data);
-	isCallingJava = saveIcj;
+	self->method = JNI_getStaticMethodIDOrNull(self->clazz, methodName, sign.data);
 
 	if(self->method == 0)
 	{
@@ -527,20 +503,17 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 
 		if(altType != 0)
 		{
-			(*env)->ExceptionClear(env);
+			JNI_exceptionClear();
 			Function_buildSignature(self, &sign, altType);
 	
 			elog(DEBUG1, "Obtaining method %s.%s %s", className, methodName, sign.data);
-			isCallingJava = true;
-			self->method = (*env)->GetStaticMethodID(
-							env, self->clazz, methodName, sign.data);
-			isCallingJava = saveIcj;
+			self->method = JNI_getStaticMethodIDOrNull(self->clazz, methodName, sign.data);
 	
 			if(self->method != 0)
 				self->returnType = realRetType;
 		}
 		if(self->method == 0)
-			PgObject_throwMemberError(env, self->clazz, methodName, origSign, true, true);
+			PgObject_throwMemberError(self->clazz, methodName, origSign, true, true);
 
 		if(sign.data != origSign)
 			pfree(origSign);
@@ -549,21 +522,21 @@ static void Function_init(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 	pfree(className);
 }
 
-static Function Function_create(JNIEnv* env, PG_FUNCTION_ARGS)
+static Function Function_create(PG_FUNCTION_ARGS)
 {
 	Function self = (Function)PgObjectClass_allocInstance(s_FunctionClass, TopMemoryContext);
-	Function_init(self, env, fcinfo);
+	Function_init(self, fcinfo);
 	return self;
 }
 
-Function Function_getFunction(JNIEnv* env, PG_FUNCTION_ARGS)
+Function Function_getFunction(PG_FUNCTION_ARGS)
 {
 	Oid funcOid = fcinfo->flinfo->fn_oid;
 	Function func = (Function)HashMap_getByOid(s_funcMap, funcOid);
 	if(func == 0)
 	{
 		PgObject old;
-		func = Function_create(env, fcinfo);
+		func = Function_create(fcinfo);
 		old = HashMap_putByOid(s_funcMap, funcOid, func);
 		if(old != 0)
 		{
@@ -577,13 +550,13 @@ Function Function_getFunction(JNIEnv* env, PG_FUNCTION_ARGS)
 	return func;
 }
 
-Datum Function_invoke(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
+Datum Function_invoke(Function self, PG_FUNCTION_ARGS)
 {
 	Datum retVal;
 	int32 top = self->numParams;
 
 	fcinfo->isnull = false;
-	currentCallContext->function = self;
+	currentInvocation->function = self;
 	if(top > 0)
 	{
 		int32   idx;
@@ -595,7 +568,7 @@ Datum Function_invoke(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 		 * connection must be dropped since its parent context is wrong.
 		 */
 		if(self->isMultiCall && SRF_IS_FIRSTCALL())
-			Backend_assertDisconnect();
+			Invocation_assertConnect();
 
 		args  = (jvalue*)palloc(top * sizeof(jvalue));
 		if(self->returnComplex)
@@ -614,46 +587,41 @@ Datum Function_invoke(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 				 */
 				args[idx].j = 0L;
 			else
-				args[idx] = Type_coerceDatum(
-								types[idx], env, PG_GETARG_DATUM(idx));
+				args[idx] = Type_coerceDatum(types[idx], PG_GETARG_DATUM(idx));
 		}
 
-		retVal = Type_invoke(invokerType,
-			env, self->clazz, self->method, args, fcinfo);
-	
+		retVal = Type_invoke(invokerType, self->clazz, self->method, args, fcinfo);
 		pfree(args);
 	}
 	else
 	{
-		retVal = Type_invoke(self->returnType,
-			env, self->clazz, self->method, NULL, fcinfo);
+		retVal = Type_invoke(self->returnType, self->clazz, self->method, NULL, fcinfo);
 	}
-
 	return retVal;
 }
 
-Datum Function_invokeTrigger(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
+Datum Function_invokeTrigger(Function self, PG_FUNCTION_ARGS)
 {
 	jvalue arg;
 	Datum  ret;
-	
-	arg.l = TriggerData_create(env, (TriggerData*)fcinfo->context);
+
+	arg.l = TriggerData_create((TriggerData*)fcinfo->context);
 	if(arg.l == 0)
 		return 0;
 
-	currentCallContext->function = self;
-	Type_invoke(self->returnType, env, self->clazz, self->method, &arg, fcinfo);
+	currentInvocation->function = self;
+	Type_invoke(self->returnType, self->clazz, self->method, &arg, fcinfo);
 
 	fcinfo->isnull = false;
-	if((*env)->ExceptionCheck(env))
+	if(JNI_exceptionCheck())
 		ret = 0;
 	else
 	{
 		/* A new Tuple may or may not be created here. If it is, ensure that
 		 * it is created in the upper SPI context.
 		 */
-		MemoryContext currCtx = MemoryContext_switchToUpperContext();
-		ret = TriggerData_getTriggerReturnTuple(env, arg.l, &fcinfo->isnull);
+		MemoryContext currCtx = Invocation_switchToUpperContext();
+		ret = PointerGetDatum(TriggerData_getTriggerReturnTuple(arg.l, &fcinfo->isnull));
 
 		/* Triggers are not allowed to set the fcinfo->isnull, even when
 		 * they return null.
@@ -663,7 +631,7 @@ Datum Function_invokeTrigger(Function self, JNIEnv* env, PG_FUNCTION_ARGS)
 		MemoryContextSwitchTo(currCtx);
 	}
 
-	(*env)->DeleteLocalRef(env, arg.l);
+	JNI_deleteLocalRef(arg.l);
 	return ret;
 }
 
@@ -672,8 +640,8 @@ bool Function_isCurrentReadOnly(void)
 	/* function will be 0 during resolve of class and java function. At
 	 * that time, no updates are allowed (or needed).
 	 */
-	return (currentCallContext->function == 0)
+	return (currentInvocation->function == 0)
 		? true
-		: currentCallContext->function->readOnly;
+		: currentInvocation->function->readOnly;
 }
 
