@@ -9,20 +9,20 @@
 #include "org_postgresql_pljava_internal_SPI.h"
 #include "pljava/backports.h"
 #include "pljava/SPI.h"
-#include "pljava/Backend.h"
+#include "pljava/Invocation.h"
 #include "pljava/Exception.h"
-#include "pljava/MemoryContext.h"
 #include "pljava/type/String.h"
-#include "pljava/type/SPITupleTable.h"
+#include "pljava/type/TupleTable.h"
 
-Datum SPI_initialize(PG_FUNCTION_ARGS)
+Savepoint* infant = 0;
+
+extern void SPI_initialize(void);
+void SPI_initialize(void)
 {
-	JNIEnv* env = (JNIEnv*)PG_GETARG_POINTER(0);
-
 	JNINativeMethod methods[] = {
 		{
 		"_exec",
-	  	"(Ljava/lang/String;I)I",
+	  	"(JLjava/lang/String;I)I",
 	  	Java_org_postgresql_pljava_internal_SPI__1exec
 		},
 		{
@@ -37,13 +37,12 @@ Datum SPI_initialize(PG_FUNCTION_ARGS)
 		},
 		{
 		"_getTupTable",
-		"()Lorg/postgresql/pljava/internal/SPITupleTable;",
+		"()Lorg/postgresql/pljava/internal/TupleTable;",
 		Java_org_postgresql_pljava_internal_SPI__1getTupTable
 		},
 		{ 0, 0, 0 }};
 
-	PgObject_registerNatives(env, "org/postgresql/pljava/internal/SPI", methods);
-	PG_RETURN_VOID();
+	PgObject_registerNatives("org/postgresql/pljava/internal/SPI", methods);
 }
 
 /****************************************
@@ -52,37 +51,36 @@ Datum SPI_initialize(PG_FUNCTION_ARGS)
 /*
  * Class:     org_postgresql_pljava_internal_SPI
  * Method:    _exec
- * Signature: (Ljava/lang/String;I)I
+ * Signature: (JLjava/lang/String;I)I
  */
 JNIEXPORT jint JNICALL
-Java_org_postgresql_pljava_internal_SPI__1exec(JNIEnv* env, jclass cls, jstring cmd, jint count)
+Java_org_postgresql_pljava_internal_SPI__1exec(JNIEnv* env, jclass cls, jlong threadId, jstring cmd, jint count)
 {
-	char* command;
-	jint result;
-	PLJAVA_ENTRY_FENCE(0)
+	jint result = 0;
 
-	command = String_createNTS(env, cmd);
-	if(command == 0)
-		return 0;
-
-	result = 0;
-	Backend_pushJavaFrame(env);
-	PG_TRY();
+	BEGIN_NATIVE
+	char* command = String_createNTS(cmd);
+	if(command != 0)
 	{
-		Backend_assertConnect();
-		result = (jint)SPI_exec(command, (int)count);
-		if(result < 0)
-			Exception_throwSPI(env, "exec", result);
-
-		Backend_popJavaFrame(env);
-		pfree(command);
+		STACK_BASE_VARS
+		STACK_BASE_PUSH(threadId)
+		PG_TRY();
+		{
+			Invocation_assertConnect();
+			result = (jint)SPI_exec(command, (int)count);
+			if(result < 0)
+				Exception_throwSPI("exec", result);
+	
+			pfree(command);
+		}
+		PG_CATCH();
+		{
+			Exception_throw_ERROR("SPI_exec");
+		}
+		PG_END_TRY();
+		STACK_BASE_POP()
 	}
-	PG_CATCH();
-	{
-		Backend_popJavaFrame(env);
-		Exception_throw_ERROR(env, "SPI_exec");
-	}
-	PG_END_TRY();
+	END_NATIVE	
 	return result;
 }
 
@@ -111,16 +109,25 @@ Java_org_postgresql_pljava_internal_SPI__1getResult(JNIEnv* env, jclass cls)
 /*
  * Class:     org_postgresql_pljava_internal_SPI
  * Method:    _getTupTable
- * Signature: ()Lorg/postgresql/pljava/internal/SPITupleTable;
+ * Signature: ()Lorg/postgresql/pljava/internal/TupleTable;
  */
 JNIEXPORT jobject JNICALL
 Java_org_postgresql_pljava_internal_SPI__1getTupTable(JNIEnv* env, jclass cls)
 {
-	PLJAVA_ENTRY_FENCE(0)
-	return SPITupleTable_create(env, SPI_tuptable);
+	jobject tupleTable = 0;
+	SPITupleTable* tts = SPI_tuptable;
+
+	if(tts != 0)
+	{
+		BEGIN_NATIVE
+		tupleTable = TupleTable_create(tts);
+		SPI_freetuptable(tts);
+		SPI_tuptable = 0;
+		END_NATIVE
+	}
+	return tupleTable;
 }
 
-#if (PGSQL_MAJOR_VER >= 8)
 static void assertXid(SubTransactionId xid)
 {
 	if(xid != GetCurrentSubTransactionId())
@@ -136,15 +143,14 @@ static void assertXid(SubTransactionId xid)
 
 Savepoint* SPI_setSavepoint(const char* name)
 {
-	/* We let the savepoint live in the current MemoryContext. It will be released
-	 * or rolled back even if the creator forgets about it.
-	 */
 	Savepoint* sp = (Savepoint*)palloc(sizeof(Savepoint) + strlen(name));
-	Backend_assertConnect();
-	BeginInternalSubTransaction((char*)name);
-	sp->nestingLevel = GetCurrentTransactionNestLevel();
-	sp->xid = GetCurrentSubTransactionId();
+	Invocation_assertConnect();
+	sp->nestingLevel = GetCurrentTransactionNestLevel() + 1;
 	strcpy(sp->name, name);
+	infant = sp;
+	BeginInternalSubTransaction(sp->name);
+	infant = 0;
+	sp->xid = GetCurrentSubTransactionId();
 	return sp;
 }
 
@@ -174,4 +180,13 @@ void SPI_rollbackSavepoint(Savepoint* sp)
 	SPI_restore_connection();
 	pfree(sp);
 }
+
+#if (PGSQL_MAJOR_VER == 8 && PGSQL_MINOR_VER < 1)
+char* SPI_getnspname(Relation rel)
+{
+	return get_namespace_name(RelationGetNamespace(rel));
+}
+
 #endif
+
+
