@@ -7,7 +7,7 @@
  * @author Thomas Hallgren
  */
 #include "pljava/type/Type_priv.h"
-#include "pljava/MemoryContext.h"
+#include "pljava/Invocation.h"
 
 static Type s_double;	/* Primitive (scalar) type */
 static TypeClass s_doubleClass;
@@ -21,27 +21,21 @@ static jmethodID s_Double_doubleValue;
 /*
  * double primitive type.
  */
-static Datum _double_invoke(Type self, JNIEnv* env, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+static Datum _double_invoke(Type self, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
 {
-	jdouble dv;
-	MemoryContext currCtx;
-	Datum ret;
-	bool saveicj = isCallingJava;
-	isCallingJava = true;
-	dv = (*env)->CallStaticDoubleMethodA(env, cls, method, args);
-	isCallingJava = saveicj;
-	
+	jdouble dv = JNI_callStaticDoubleMethodA(cls, method, args);
+
 	/* Since we don't know if 64 bit quantities are passed by reference or
 	 * by value, we have to make sure that the correct context is used if
 	 * it's the former.
 	 */
-	currCtx = MemoryContext_switchToUpperContext();
-	ret = Float8GetDatum(dv);
+	MemoryContext currCtx = Invocation_switchToUpperContext();
+	Datum ret = Float8GetDatum(dv);
 	MemoryContextSwitchTo(currCtx);
 	return ret;
 }
 
-static jvalue _double_coerceDatum(Type self, JNIEnv* env, Datum arg)
+static jvalue _double_coerceDatum(Type self, Datum arg)
 {
 	jvalue result;
 	result.d = DatumGetFloat8(arg);
@@ -61,20 +55,16 @@ static bool _Double_canReplace(Type self, Type other)
 	return self->m_class == other->m_class || other->m_class == s_doubleClass;
 }
 
-static jvalue _Double_coerceDatum(Type self, JNIEnv* env, Datum arg)
+static jvalue _Double_coerceDatum(Type self, Datum arg)
 {
 	jvalue result;
-	result.l = PgObject_newJavaObject(env, s_Double_class, s_Double_init, DatumGetFloat8(arg));
+	result.l = JNI_newObject(s_Double_class, s_Double_init, DatumGetFloat8(arg));
 	return result;
 }
 
-static Datum _Double_coerceObject(Type self, JNIEnv* env, jobject doubleObj)
+static Datum _Double_coerceObject(Type self, jobject doubleObj)
 {
-	jdouble dv;
-	bool saveicj = isCallingJava;
-	isCallingJava = true;
-	dv = (*env)->CallDoubleMethod(env, doubleObj, s_Double_doubleValue);
-	isCallingJava = saveicj;
+	jdouble dv = JNI_callDoubleMethod(doubleObj, s_Double_doubleValue);
 	return Float8GetDatum(dv);
 }
 
@@ -85,20 +75,13 @@ static Type Double_obtain(Oid typeId)
 
 /* Make this datatype available to the postgres system.
  */
-extern Datum Double_initialize(PG_FUNCTION_ARGS);
-PG_FUNCTION_INFO_V1(Double_initialize);
-Datum Double_initialize(PG_FUNCTION_ARGS)
+extern void Double_initialize(void);
+void Double_initialize(void)
 {
-	JNIEnv* env = (JNIEnv*)PG_GETARG_POINTER(0);
+	s_Double_class = JNI_newGlobalRef(PgObject_getJavaClass("java/lang/Double"));
+	s_Double_init = PgObject_getJavaMethod(s_Double_class, "<init>", "(D)V");
 
-	s_Double_class = (*env)->NewGlobalRef(
-				env, PgObject_getJavaClass(env, "java/lang/Double"));
-
-	s_Double_init = PgObject_getJavaMethod(
-				env, s_Double_class, "<init>", "(D)V");
-
-	s_Double_doubleValue = PgObject_getJavaMethod(
-				env, s_Double_class, "doubleValue", "()D");
+	s_Double_doubleValue = PgObject_getJavaMethod(s_Double_class, "doubleValue", "()D");
 
 	s_DoubleClass = TypeClass_alloc("type.Double");
 	s_DoubleClass->canReplaceType = _Double_canReplace;
@@ -120,5 +103,4 @@ Datum Double_initialize(PG_FUNCTION_ARGS)
 	Type_registerPgType(FLOAT8OID, double_obtain);
 	Type_registerJavaType("double", double_obtain);
 	Type_registerJavaType("java.lang.Double", Double_obtain);
-	PG_RETURN_VOID();
 }
