@@ -7,7 +7,8 @@
  * @author Thomas Hallgren
  */
 #include "pljava/type/String_priv.h"
-#include "pljava/MemoryContext.h"
+#include "pljava/type/TupleDesc.h"
+#include "pljava/Invocation.h"
 #include "pljava/HashMap.h"
 #include "pljava/SPI.h"
 
@@ -20,14 +21,14 @@ bool Type_canReplaceType(Type self, Type other)
 	return self->m_class->canReplaceType(self, other);
 }
 
-jvalue Type_coerceDatum(Type self, JNIEnv* env, Datum value)
+jvalue Type_coerceDatum(Type self, Datum value)
 {
-	return self->m_class->coerceDatum(self, env, value);
+	return self->m_class->coerceDatum(self, value);
 }
 
-Datum Type_coerceObject(Type self, JNIEnv* env, jobject object)
+Datum Type_coerceObject(Type self, jobject object)
 {
-	return self->m_class->coerceObject(self, env, object);
+	return self->m_class->coerceObject(self, object);
 }
 
 const char* Type_getJavaTypeName(Type self)
@@ -50,9 +51,14 @@ Oid Type_getOid(Type self)
 	return self->m_oid;
 }
 
-Datum Type_invoke(Type self, JNIEnv* env, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+TupleDesc Type_getTupleDesc(Type self, PG_FUNCTION_ARGS)
 {
-	return self->m_class->invoke(self, env, cls, method, args, fcinfo);
+	return self->m_class->getTupleDesc(self, fcinfo);
+}
+
+Datum Type_invoke(Type self, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+{
+	return self->m_class->invoke(self, cls, method, args, fcinfo);
 }
 
 bool Type_isPrimitive(Type self)
@@ -111,133 +117,132 @@ Type Type_fromOid(Oid typeId)
 	return type;
 }
 
+Type Type_objectTypeFromOid(Oid typeId)
+{
+	Type type = Type_fromOid(typeId);
+	Type objectType = type->m_class->objectType;
+	return (objectType == 0) ? type : objectType;
+}
+
 bool _Type_canReplaceType(Type self, Type other)
 {
 	return self->m_class == other->m_class;
 }
 
-Datum _Type_invoke(Type self, JNIEnv* env, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+Datum _Type_invoke(Type self, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
 {
-	jobject value;
 	MemoryContext currCtx;
 	Datum ret;
-	bool saveicj = isCallingJava;
-	
-	isCallingJava = true;
-	value = (*env)->CallStaticObjectMethodA(env, cls, method, args);
-	isCallingJava = saveicj;
+	jobject value = JNI_callStaticObjectMethodA(cls, method, args);
 	if(value == 0)
 	{
 		fcinfo->isnull = true;
 		return 0;
 	}
-	
+
 	/* The return value cannot be created in the current context since it
 	 * goes out of scope when SPI_finish is called.
 	 */
-	currCtx = MemoryContext_switchToUpperContext();
-	ret = self->m_class->coerceObject(self, env, value);
+	currCtx = Invocation_switchToUpperContext();
+	ret = self->m_class->coerceObject(self, value);
 	MemoryContextSwitchTo(currCtx);
-	(*env)->DeleteLocalRef(env, value);
+	JNI_deleteLocalRef(value);
 	return ret;
 }
 
+TupleDesc _Type_getTupleDesc(Type self, PG_FUNCTION_ARGS)
+{
+	ereport(ERROR,
+		(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+		 errmsg("Type is not associated with a record")));
+	return 0;	/* Keep compiler happy */
+}
+
 /*
  * Shortcuts to initializers of known types
  */
-extern Datum Void_initialize(PG_FUNCTION_ARGS);
-extern Datum Boolean_initialize(PG_FUNCTION_ARGS);
-extern Datum Byte_initialize(PG_FUNCTION_ARGS);
-extern Datum Short_initialize(PG_FUNCTION_ARGS);
-extern Datum Integer_initialize(PG_FUNCTION_ARGS);
-extern Datum Long_initialize(PG_FUNCTION_ARGS);
-extern Datum Float_initialize(PG_FUNCTION_ARGS);
-extern Datum Double_initialize(PG_FUNCTION_ARGS);
-extern Datum BigDecimal_initialize(PG_FUNCTION_ARGS);
+extern void Void_initialize(void);
+extern void Boolean_initialize(void);
+extern void Byte_initialize(void);
+extern void Short_initialize(void);
+extern void Integer_initialize(void);
+extern void Long_initialize(void);
+extern void Float_initialize(void);
+extern void Double_initialize(void);
+extern void BigDecimal_initialize(void);
 
-extern Datum Date_initialize(PG_FUNCTION_ARGS);
-extern Datum Time_initialize(PG_FUNCTION_ARGS);
-extern Datum Timestamp_initialize(PG_FUNCTION_ARGS);
+extern void Date_initialize(void);
+extern void Time_initialize(void);
+extern void Timestamp_initialize(void);
 
-extern Datum Oid_initialize(PG_FUNCTION_ARGS);
-extern Datum AclId_initialize(PG_FUNCTION_ARGS);
-extern Datum ErrorData_initialize(PG_FUNCTION_ARGS);
+extern void Oid_initialize(void);
+extern void AclId_initialize(void);
+extern void ErrorData_initialize(void);
+extern void LargeObject_initialize(void);
 
-extern Datum String_initialize(PG_FUNCTION_ARGS);
-extern Datum byte_array_initialize(PG_FUNCTION_ARGS);
+extern void String_initialize(void);
+extern void byte_array_initialize(void);
 
-extern Datum NativeStruct_initialize(PG_FUNCTION_ARGS);
-extern Datum ExecutionPlan_initialize(PG_FUNCTION_ARGS);
-extern Datum Portal_initialize(PG_FUNCTION_ARGS);
-extern Datum Relation_initialize(PG_FUNCTION_ARGS);
-extern Datum SPITupleTable_initialize(PG_FUNCTION_ARGS);
-extern Datum TriggerData_initialize(PG_FUNCTION_ARGS);
-extern Datum Tuple_initialize(PG_FUNCTION_ARGS);
-extern Datum TupleDesc_initialize(PG_FUNCTION_ARGS);
-extern Datum TupleTable_initialize(PG_FUNCTION_ARGS);
-extern Datum TupleTableSlot_initialize(PG_FUNCTION_ARGS);
-extern Datum HeapTupleHeader_initialize(PG_FUNCTION_ARGS);
+extern void JavaWrapper_initialize(void);
+extern void ExecutionPlan_initialize(void);
+extern void Portal_initialize(void);
+extern void Relation_initialize(void);
+extern void TriggerData_initialize(void);
+extern void Tuple_initialize(void);
+extern void TupleDesc_initialize(void);
+extern void TupleTable_initialize(void);
+extern void HeapTupleHeader_initialize(void);
 
-extern Datum SingleRowReader_initialize(PG_FUNCTION_ARGS);
-extern Datum SingleTupleReader_initialize(PG_FUNCTION_ARGS);
-extern Datum SingleRowWriter_initialize(PG_FUNCTION_ARGS);
-extern Datum ValueSetProvider_initialize(PG_FUNCTION_ARGS);
-extern Datum ResultSetProvider_initialize(PG_FUNCTION_ARGS);
-extern Datum Savepoint_initialize(PG_FUNCTION_ARGS);
+extern void SingleTupleReader_initialize(void);
+extern void SingleRowWriter_initialize(void);
+extern void ValueSetProvider_initialize(void);
+extern void ResultSetProvider_initialize(void);
 
-/* Make this datatype available to the postgres system.
- */
-PG_FUNCTION_INFO_V1(Type_initialize);
-
-Datum Type_initialize(PG_FUNCTION_ARGS)
+extern void Type_initialize(void);
+void Type_initialize(void)
 {
 	s_typeByOid          = HashMap_create(59, TopMemoryContext);
 	s_obtainerByOid      = HashMap_create(59, TopMemoryContext);
 	s_obtainerByJavaName = HashMap_create(59, TopMemoryContext);
 
-	String_initialize(fcinfo);
+	String_initialize();
 	
-	Void_initialize(fcinfo);
-	Boolean_initialize(fcinfo);
-	Byte_initialize(fcinfo);
-	Short_initialize(fcinfo);
-	Integer_initialize(fcinfo);
-	Long_initialize(fcinfo);
-	Float_initialize(fcinfo);
-	Double_initialize(fcinfo);
+	Void_initialize();
+	Boolean_initialize();
+	Byte_initialize();
+	Short_initialize();
+	Integer_initialize();
+	Long_initialize();
+	Float_initialize();
+	Double_initialize();
 
-	BigDecimal_initialize(fcinfo);
+	BigDecimal_initialize();
 
-	Date_initialize(fcinfo);
-	Time_initialize(fcinfo);
-	Timestamp_initialize(fcinfo);
+	Date_initialize();
+	Time_initialize();
+	Timestamp_initialize();
 
-	Oid_initialize(fcinfo);
-	AclId_initialize(fcinfo);
-	ErrorData_initialize(fcinfo);
+	Oid_initialize();
+	AclId_initialize();
+	ErrorData_initialize();
+	LargeObject_initialize();
 
-	byte_array_initialize(fcinfo);
+	byte_array_initialize();
 
-	NativeStruct_initialize(fcinfo);
-	ExecutionPlan_initialize(fcinfo);
-	Portal_initialize(fcinfo);
-	TriggerData_initialize(fcinfo);
-	Relation_initialize(fcinfo);
-	SPITupleTable_initialize(fcinfo);
-	TupleDesc_initialize(fcinfo);
-	Tuple_initialize(fcinfo);
-	TupleTable_initialize(fcinfo);
-	TupleTableSlot_initialize(fcinfo);
-	HeapTupleHeader_initialize(fcinfo);
+	JavaWrapper_initialize();
+	ExecutionPlan_initialize();
+	Portal_initialize();
+	TriggerData_initialize();
+	Relation_initialize();
+	TupleDesc_initialize();
+	Tuple_initialize();
+	TupleTable_initialize();
+	HeapTupleHeader_initialize();
 
-	SingleRowReader_initialize(fcinfo);
-	SingleTupleReader_initialize(fcinfo);
-	SingleRowWriter_initialize(fcinfo);
-	ValueSetProvider_initialize(fcinfo);
-	ResultSetProvider_initialize(fcinfo);
-	Savepoint_initialize(fcinfo);
-
-	PG_RETURN_VOID();
+	SingleTupleReader_initialize();
+	SingleRowWriter_initialize();
+	ValueSetProvider_initialize();
+	ResultSetProvider_initialize();
 }
 
 /*
@@ -256,9 +261,10 @@ TypeClass TypeClass_alloc2(const char* typeName, Size classSize, Size instanceSi
 	self->javaTypeName   = "";
 	self->objectType     = 0;
 	self->canReplaceType = _Type_canReplaceType;
-	self->coerceDatum    = (jvalue (*)(Type, JNIEnv*, Datum))_PgObject_pureVirtualCalled;
-	self->coerceObject   = (Datum (*)(Type, JNIEnv*, jobject))_PgObject_pureVirtualCalled;
+	self->coerceDatum    = (jvalue (*)(Type, Datum))_PgObject_pureVirtualCalled;
+	self->coerceObject   = (Datum (*)(Type, jobject))_PgObject_pureVirtualCalled;
 	self->invoke         = _Type_invoke;
+	self->getTupleDesc   = _Type_getTupleDesc;
 	return self;
 }
 
