@@ -12,13 +12,11 @@
 
 #include "pljava/type/Type_priv.h"
 #include "pljava/HashMap.h"
-#include "pljava/MemoryContext.h"
+#include "pljava/Invocation.h"
 
-#ifndef GCJ /* Bug libgcj/15001 */
 static jclass s_Iterator_class;
 static jmethodID s_Iterator_hasNext;
 static jmethodID s_Iterator_next;
-#endif
 
 static TypeClass s_ValueSetProviderClass;
 static HashMap s_cache;	/* Cached by element type */
@@ -27,35 +25,24 @@ typedef struct
 {
 	jobject iterator;
 	Type    elementType;
-#ifdef GCJ /* Bug libgcj/15001 */
-	jmethodID next;
-	jmethodID hasNext;
-#endif
 } CallContextData;
 
-static Datum _ValueSetProvider_invoke(Type self, JNIEnv* env, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
+static Datum _ValueSetProvider_invoke(Type self, jclass cls, jmethodID method, jvalue* args, PG_FUNCTION_ARGS)
 {
 	bool hasRow;
 	CallContextData* ctxData;
 	FuncCallContext* context;
-	bool saveicj = isCallingJava;
 
 	/* stuff done only on the first call of the function
 	 */
 	if(SRF_IS_FIRSTCALL())
 	{
-#ifdef GCJ /* Bug libgcj/15001 */
-		jclass rsClass;
-#endif
 		MemoryContext currCtx;
-		jobject tmp;
 
 		/* Call the declared Java function. It returns the Iterator
 		 * that later is used once for each row that should be obtained.
 		 */
-		isCallingJava = true;
-		tmp = (*env)->CallStaticObjectMethodA(env, cls, method, args);
-		isCallingJava = saveicj;
+		jobject tmp = JNI_callStaticObjectMethodA(cls, method, args);
 
 		/* create a function context for cross-call persistence
 		 */
@@ -77,17 +64,9 @@ static Datum _ValueSetProvider_invoke(Type self, JNIEnv* env, jclass cls, jmetho
 		MemoryContextSwitchTo(currCtx);
 
 		context->user_fctx = ctxData;
-		ctxData->iterator = (*env)->NewGlobalRef(env, tmp);
+		ctxData->iterator = JNI_newGlobalRef(tmp);
 		ctxData->elementType = Type_fromOid(Type_getOid(self));
-#ifdef GCJ /* Bug libgcj/15001 */
-		rsClass = (*env)->GetObjectClass(env, tmp);
-		ctxData->hasNext = PgObject_getJavaMethod(
-				env, rsClass, "hasNext", "()Z");
-		ctxData->next = PgObject_getJavaMethod(
-				env, rsClass, "next", "()Ljava/lang/Object;");
-		(*env)->DeleteLocalRef(env, rsClass);
-#endif
-		(*env)->DeleteLocalRef(env, tmp);
+		JNI_deleteLocalRef(tmp);
 	}
 
 	context = SRF_PERCALL_SETUP();
@@ -96,54 +75,37 @@ static Datum _ValueSetProvider_invoke(Type self, JNIEnv* env, jclass cls, jmetho
 	/* Obtain next row using the RowProvider as a parameter to the
 	 * Iterator.assignRowValues method.
 	 */
-	isCallingJava = true;
-	hasRow = ((*env)->CallBooleanMethod(
-			env, ctxData->iterator,
-#ifdef GCJ /* Bug libgcj/15001 */
-			ctxData->hasNext
-#else
-			s_Iterator_hasNext
-#endif
-			) == JNI_TRUE);
-	isCallingJava = saveicj;
+	hasRow = (JNI_callBooleanMethod(ctxData->iterator, s_Iterator_hasNext) == JNI_TRUE);
 
 	if(hasRow)
 	{
 		MemoryContext currCtx;
 		Datum result;
-		jobject tmp;
-		
-		isCallingJava = true;
-#ifdef GCJ /* Bug libgcj/15001 */
-		tmp = (*env)->CallObjectMethod(env, ctxData->iterator, ctxData->next);
-#else
-		tmp = (*env)->CallObjectMethod(env, ctxData->iterator, s_Iterator_next);
-#endif
-		isCallingJava = saveicj;
+		jobject tmp = JNI_callObjectMethod(ctxData->iterator, s_Iterator_next);
 
 		/* The element must be coerced using the return value context
 		 */
-		currCtx = MemoryContext_switchToUpperContext();
-		result = Type_coerceObject(ctxData->elementType, env, tmp);
+		currCtx = Invocation_switchToUpperContext();
+		result = Type_coerceObject(ctxData->elementType, tmp);
 		MemoryContextSwitchTo(currCtx);
 		SRF_RETURN_NEXT(context, result);
 	}
 
 	/* This is the end of the set.
 	 */
-	(*env)->DeleteGlobalRef(env, ctxData->iterator);
+	JNI_deleteGlobalRef(ctxData->iterator);
 	pfree(ctxData);
 	SRF_RETURN_DONE(context);
 }
 
-static jvalue _ValueSetProvider_coerceDatum(Type self, JNIEnv* env, Datum nothing)
+static jvalue _ValueSetProvider_coerceDatum(Type self, Datum nothing)
 {
 	jvalue result;
 	result.j = 0L;
 	return result;
 }
 
-static Datum _ValueSetProvider_coerceObject(Type self, JNIEnv* env, jobject nothing)
+static Datum _ValueSetProvider_coerceObject(Type self, jobject nothing)
 {
 	return 0;
 }
@@ -164,19 +126,12 @@ static Type ValueSetProvider_obtain(Oid typeId)
 
 /* Make this datatype available to the postgres system.
  */
-extern Datum ValueSetProvider_initialize(PG_FUNCTION_ARGS);
-PG_FUNCTION_INFO_V1(ValueSetProvider_initialize);
-Datum ValueSetProvider_initialize(PG_FUNCTION_ARGS)
+extern void ValueSetProvider_initialize(void);
+void ValueSetProvider_initialize(void)
 {
-#ifndef GCJ /* Bug libgcj/15001 */
-	JNIEnv* env = (JNIEnv*)PG_GETARG_POINTER(0);
-	s_Iterator_class = (*env)->NewGlobalRef(
-				env, PgObject_getJavaClass(env, "java/util/Iterator"));
-	s_Iterator_hasNext = PgObject_getJavaMethod(
-				env, s_Iterator_class, "hasNext", "()Z");
-	s_Iterator_next = PgObject_getJavaMethod(
-				env, s_Iterator_class, "next", "()Ljava/lang/Object;");
-#endif
+	s_Iterator_class = JNI_newGlobalRef(PgObject_getJavaClass("java/util/Iterator"));
+	s_Iterator_hasNext = PgObject_getJavaMethod(s_Iterator_class, "hasNext", "()Z");
+	s_Iterator_next = PgObject_getJavaMethod(s_Iterator_class, "next", "()Ljava/lang/Object;");
 
 	s_cache = HashMap_create(13, TopMemoryContext);
 
@@ -188,5 +143,4 @@ Datum ValueSetProvider_initialize(PG_FUNCTION_ARGS)
 	s_ValueSetProviderClass->coerceObject = _ValueSetProvider_coerceObject;
 
 	Type_registerJavaType("java.util.Iterator", ValueSetProvider_obtain);
-	PG_RETURN_VOID();
 }
