@@ -121,7 +121,7 @@ public class Deployer
 	public static void printUsage()
 	{
 		PrintStream out = System.err;
-		out.println("usage: java -jar deploy.jar");
+		out.println("usage: java org.postgresql.pljava.deploy.Deployer");
 		out.println("    {-install | -uninstall | -reinstall}");
 		out.println("    [ -host <hostName>     ]    # default is localhost");
 		out.println("    [ -port <portNumber>   ]    # default is blank");
@@ -272,7 +272,7 @@ public class Deployer
 			if(cmd == CMD_INSTALL || cmd == CMD_REINSTALL)
 			{
 				deployer.createSQLJSchema();
-				deployer.initJavaHandler(cygwin);
+				deployer.initJavaHandlers(cygwin);
 				deployer.initializeSQLJSchema();
 			}
 			c.close();
@@ -317,6 +317,7 @@ public class Deployer
 		try
 		{
 			stmt.execute("DROP LANGUAGE java CASCADE");
+			stmt.execute("DROP LANGUAGE javaU CASCADE");
 		}
 		catch(SQLException e)
 		{
@@ -344,7 +345,8 @@ public class Deployer
 			"CREATE TABLE sqlj.jar_repository(" +
 			"	jarId		SERIAL PRIMARY KEY," +
 			"	jarName		VARCHAR(100) UNIQUE NOT NULL," +
-			"   jarOrigin   VARCHAR(500) NOT NULL," +
+			"	jarOrigin   VARCHAR(500) NOT NULL," +
+			"	jarOwner	INT NOT NULL," +
 			"   deploymentDesc INT" +
 		")");
 		stmt.execute("GRANT SELECT ON sqlj.jar_repository TO public");
@@ -392,6 +394,19 @@ public class Deployer
 			"	AS 'org.postgresql.pljava.management.Commands.removeJar'" +
 			"	LANGUAGE java SECURITY DEFINER");
 
+		// Not proposed, but very useful if you want to send the image over
+		// your JDBC connection.
+		//
+		stmt.execute(
+			"CREATE FUNCTION sqlj.install_jar(BYTEA, VARCHAR, BOOLEAN) RETURNS void" +
+			"	AS 'org.postgresql.pljava.management.Commands.installJar'" +
+			"	LANGUAGE java SECURITY DEFINER");
+
+		stmt.execute(
+			"CREATE FUNCTION sqlj.replace_jar(BYTEA, VARCHAR, BOOLEAN) RETURNS void" +
+			"	AS 'org.postgresql.pljava.management.Commands.replaceJar'" +
+			"	LANGUAGE java SECURITY DEFINER");
+
 		// This function is not as proposed. It's more Java'ish. The proposal
 		// using sqlj.alter_jar_path is in my opinion bloated and will not be
 		// well received in the Java community. Luckily, the support is suggested
@@ -410,7 +425,7 @@ public class Deployer
 		stmt.close();
 	}
 
-	public void initJavaHandler(boolean cygwin)
+	public void initJavaHandlers(boolean cygwin)
 	throws SQLException
 	{
 		Statement stmt = m_connection.createStatement();
@@ -421,6 +436,14 @@ public class Deployer
 			" LANGUAGE C");
 
 		stmt.execute("CREATE TRUSTED LANGUAGE java HANDLER sqlj.java_call_handler");
+
+		stmt.execute(
+			"CREATE FUNCTION sqlj.javau_call_handler()" +
+			" RETURNS language_handler" +
+			" AS '" + (cygwin ? "" : "lib") + "pljava'" +
+			" LANGUAGE C");
+
+		stmt.execute("CREATE LANGUAGE javaU HANDLER sqlj.javau_call_handler");
 		stmt.close();
 	}
 }
