@@ -22,6 +22,22 @@ import java.sql.SQLException;
  */
 public interface Session
 {
+	/**
+	 * Adds the specified <code>listener</code> to the list of listeners that will
+	 * receive savepoint events. This method does nothing if the listener
+	 * was already added.
+	 * @param listener The listener to be added.
+	 */
+	void addSavepointListener(SavepointListener listener);
+
+	/**
+	 * Adds the specified <code>listener</code> to the list of listeners that will
+	 * receive transactional events. This method does nothing if the listener
+	 * was already added.
+	 * @param listener The listener to be added.
+	 */
+	void addTransactionListener(TransactionListener listener);
+
 	/**
 	 * Obtain an attribute from the current session.
 	 * @param attributeName The name of the attribute
@@ -29,6 +45,14 @@ public interface Session
 	 */
 	Object getAttribute(String attributeName);
 
+	/**
+	 * Return an object pool for the given class. The class must implement
+	 * the interface {@link PooledObject}.
+	 * @param cls
+	 * @return An object pool that pools object of the given class.
+	 */
+	ObjectPool getObjectPool(Class cls);
+
 	/**
 	 * Return the name of the effective user. If the currently
 	 * executing funciton is declared with <code>SECURITY DEFINER</code>,
@@ -64,6 +88,22 @@ public interface Session
 	 */
 	void removeAttribute(String attributeName);
 
+	/**
+	 * Removes the specified <code>listener</code> from the list of listeners that will
+	 * receive savepoint events. This method does nothing unless the listener is
+	 * found.
+	 * @param listener The listener to be removed.
+	 */
+	void removeSavepointListener(SavepointListener listener);
+
+	/**
+	 * Removes the specified <code>listener</code> from the list of listeners that will
+	 * receive transactional events. This method does nothing unless the listener is
+	 * found.
+	 * @param listener The listener to be removed.
+	 */
+	void removeTransactionListener(TransactionListener listener);
+
 	/**
 	 * Set an attribute to a value in the current session.
 	 * @param attributeName
