@@ -7,56 +7,66 @@
 package org.postgresql.pljava.internal;
 
 import java.sql.SQLException;
+import java.util.Collections;
 import java.util.Map;
 import java.util.LinkedHashMap;
 
 /**
  * The <code>ExecutionPlan</code> correspons to the execution plan obtained
  * using an internal PostgreSQL <code>SPI_prepare</code> call.
- *
+ * 
  * @author Thomas Hallgren
  */
-public class ExecutionPlan extends NativeStruct
+public class ExecutionPlan
 {
-    static final int DEFAULT_INITIAL_CAPACITY = 29;
-    static final float DEFAULT_LOAD_FACTOR = 0.75f;
+	static final int INITIAL_CACHE_CAPACITY = 29;
 
-    /**
-     * MRU cache for prepared plans.
-     */
-    static final class PlanCache extends LinkedHashMap
+	static final float CACHE_LOAD_FACTOR = 0.75f;
+
+	private long m_pointer;
+
+	/**
+	 * MRU cache for prepared plans.
+	 */
+	static final class PlanCache extends LinkedHashMap
 	{
-    	private final int m_cacheSize;
+		private final int m_cacheSize;
 
 		public PlanCache(int cacheSize)
 		{
-			super(DEFAULT_INITIAL_CAPACITY, DEFAULT_LOAD_FACTOR, true);
+			super(INITIAL_CACHE_CAPACITY, CACHE_LOAD_FACTOR, true);
 			m_cacheSize = cacheSize;
 		}
 
 		protected boolean removeEldestEntry(Map.Entry eldest)
 		{
-	        if(this.size() > m_cacheSize)
-	        {
-	    		synchronized(Backend.THREADLOCK)
+			if(this.size() <= m_cacheSize)
+				return false;
+
+			ExecutionPlan evicted = (ExecutionPlan)eldest.getValue();
+			synchronized(Backend.THREADLOCK)
+			{
+				if(evicted.m_pointer != 0)
 				{
-	    			((ExecutionPlan)eldest.getValue())._invalidate();
+					_invalidate(evicted.m_pointer);
+					evicted.m_pointer = 0;
 				}
-	        	return true;
-	        }
-	        return false;
-	    }
+			}
+			return true;
+		}
 	};
 
 	static final class PlanKey
 	{
-		private final int    m_hashCode;
+		private final int m_hashCode;
+
 		private final String m_stmt;
-		private final Oid[]  m_argTypes;
+
+		private final Oid[] m_argTypes;
 
 		PlanKey(String stmt, Oid[] argTypes)
 		{
-			m_stmt     = stmt;
+			m_stmt = stmt;
 			m_hashCode = stmt.hashCode() + 1;
 			m_argTypes = argTypes;
 		}
@@ -82,26 +92,29 @@ public class ExecutionPlan extends NativeStruct
 
 			return true;
 		}
-		
+
 		public int hashCode()
 		{
 			return m_hashCode;
 		}
 	}
 
-	private static final PlanCache s_planCache;
+	private static final Map s_planCache;
 
 	private final Object m_key;
 
 	static
 	{
 		int cacheSize = Backend.getStatementCacheSize();
-		s_planCache = new PlanCache(cacheSize < 11 ? 11 : cacheSize);
+		s_planCache = Collections.synchronizedMap(new PlanCache(cacheSize < 11
+			? 11
+			: cacheSize));
 	}
 
-	private ExecutionPlan(Object key)
+	private ExecutionPlan(Object key, long pointer)
 	{
 		m_key = key;
+		m_pointer = pointer;
 	}
 
 	/**
@@ -110,11 +123,12 @@ public class ExecutionPlan extends NativeStruct
 	public void close()
 	{
 		ExecutionPlan old = (ExecutionPlan)s_planCache.put(m_key, this);
-		if(old != null)
+		if(old != null && old.m_pointer != 0)
 		{
 			synchronized(Backend.THREADLOCK)
 			{
-				old._invalidate();
+				_invalidate(old.m_pointer);
+				old.m_pointer = 0;
 			}
 		}
 	}
@@ -122,8 +136,9 @@ public class ExecutionPlan extends NativeStruct
 	/**
 	 * Set up a cursor that will execute the plan using the internal
 	 * <code>SPI_cursor_open</code> function
-	 * @param cursorName Name of the cursor or <code>null</code> for a
-	 * system generated name.
+	 * 
+	 * @param cursorName Name of the cursor or <code>null</code> for a system
+	 *            generated name.
 	 * @param parameters Values for the parameters.
 	 * @return The <code>Portal</code> that represents the opened cursor.
 	 * @throws SQLException If the underlying native structure has gone stale.
@@ -133,47 +148,51 @@ public class ExecutionPlan extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._cursorOpen(cursorName, parameters);
+			return _cursorOpen(m_pointer, System.identityHashCode(Thread
+				.currentThread()), cursorName, parameters);
 		}
 	}
-	
+
 	/**
 	 * Checks if this <code>ExecutionPlan</code> can create a <code>Portal
-	 * </code> using {@link #cursorOpen}. This is true if the
-	 * plan contains only one regular <code>SELECT</code> query.
-	 * @return <code>true</code> if the plan can create a <code>Portal</code> 
+	 * </code>
+	 * using {@link #cursorOpen}. This is true if the plan contains only one
+	 * regular <code>SELECT</code> query.
+	 * 
+	 * @return <code>true</code> if the plan can create a <code>Portal</code>
 	 * @throws SQLException If the underlying native structure has gone stale.
 	 */
-	public boolean isCursorPlan()
-	throws SQLException
+	public boolean isCursorPlan() throws SQLException
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isCursorPlan();
+			return _isCursorPlan(m_pointer);
 		}
 	}
 
 	/**
 	 * Execute the plan using the internal <code>SPI_execp</code> function.
+	 * 
 	 * @param parameters Values for the parameters.
-	 * @param rowCount The maximum number of tuples to create. A value
-	 * of <code>rowCount</code> of zero is interpreted as no limit, i.e.,
-	 * run to completion.
+	 * @param rowCount The maximum number of tuples to create. A value of
+	 *            <code>rowCount</code> of zero is interpreted as no limit,
+	 *            i.e., run to completion.
 	 * @return One of the status codes declared in class {@link SPI}.
 	 * @throws SQLException If the underlying native structure has gone stale.
 	 */
-	public int execute(Object[] parameters, int rowCount)
-	throws SQLException
+	public int execute(Object[] parameters, int rowCount) throws SQLException
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._execute(parameters, rowCount);
+			return _execute(m_pointer, System.identityHashCode(Thread
+				.currentThread()), parameters, rowCount);
 		}
 	}
 
 	/**
-	 * Create an execution plan for a statement to be executed later using
-	 * the internal <code>SPI_prepare</code> function.
+	 * Create an execution plan for a statement to be executed later using the
+	 * internal <code>SPI_prepare</code> function.
+	 * 
 	 * @param statement The command string.
 	 * @param argTypes SQL types of argument types.
 	 * @return An execution plan for the prepared statement.
@@ -190,26 +209,25 @@ public class ExecutionPlan extends NativeStruct
 		ExecutionPlan plan = (ExecutionPlan)s_planCache.remove(key);
 		if(plan == null)
 		{
-			plan = new ExecutionPlan(key);
 			synchronized(Backend.THREADLOCK)
 			{
-				plan._prepare(statement, argTypes);
+				plan = new ExecutionPlan(key, _prepare(statement, argTypes));
 			}
 		}
 		return plan;
 	}
 
-	private native Portal _cursorOpen(String cursorName, Object[] parameters)
+	private static native Portal _cursorOpen(long pointer, long threadId,
+		String cursorName, Object[] parameters) throws SQLException;
+
+	private static native boolean _isCursorPlan(long pointer)
 	throws SQLException;
 
-	private native boolean _isCursorPlan()
+	private static native int _execute(long pointer, long threadId,
+		Object[] parameters, int rowCount) throws SQLException;
+
+	private static native long _prepare(String statement, Oid[] argTypes)
 	throws SQLException;
 
-	private native int _execute(Object[] parameters, int rowCount)
-	throws SQLException;
-
-	private native void _prepare(String statement, Oid[] argTypes)
-	throws SQLException;
-
-	private native void _invalidate();
+	private static native void _invalidate(long pointer);
 }
