@@ -14,14 +14,26 @@ import java.sql.SQLException;
  *
  * @author Thomas Hallgren
  */
-public class Portal extends NativeStruct
+public class Portal
 {
+	private long m_pointer;
+
+	Portal(long pointer)
+	{
+		m_pointer = pointer;
+	}
+
 	/**
-	 * Performs an <code>SPI_cursor_close</code>.
+	 * Invalidates this structure and frees up memory using the
+	 * internal function <code>SPI_cursor_close</code>
 	 */
 	public void close()
 	{
-		this.invalidate();
+		synchronized(Backend.THREADLOCK)
+		{
+			_close(m_pointer);
+			m_pointer = 0;
+		}
 	}
 
 	/**
@@ -33,7 +45,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._getName();
+			return _getName(m_pointer);
 		}
 	}
 
@@ -46,7 +58,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._getPortalPos();
+			return _getPortalPos(m_pointer);
 		}
 	}
 
@@ -60,7 +72,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._getTupleDesc();
+			return _getTupleDesc(m_pointer);
 		}
 	}
 
@@ -76,19 +88,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._fetch(forward, count);
-		}
-	}
-
-	/**
-	 * Invalidates this structure and frees up memory using the
-	 * internal function <code>SPI_cursor_close</code>
-	 */
-	public void invalidate()
-	{
-		synchronized(Backend.THREADLOCK)
-		{
-			this._invalidate();
+			return _fetch(m_pointer, forward, count);
 		}
 	}
 
@@ -101,7 +101,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isAtEnd();
+			return _isAtEnd(m_pointer);
 		}
 	}
 
@@ -114,7 +114,7 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isAtStart();
+			return _isAtStart(m_pointer);
 		}
 	}
 
@@ -127,10 +127,20 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isPosOverflow();
+			return _isPosOverflow(m_pointer);
 		}
 	}
 
+	/**
+	 * Checks if the portal is still active. I can be closed either explicitly
+	 * using the {@link #close()} mehtod or implicitly due to a pop of invocation
+	 * context.
+	 */
+	public boolean isValid()
+	{
+		return m_pointer != 0;
+	}
+
 	/**
 	 * Performs an <code>SPI_cursor_move</code>.
 	 * @param forward Set to <code>true</code> for forward, <code>false</code> for backward.
@@ -143,33 +153,33 @@ public class Portal extends NativeStruct
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._move(forward, count);
+			return _move(m_pointer, forward, count);
 		}
 	}
 
-	private native String _getName()
+	private static native String _getName(long pointer)
 	throws SQLException;
 
-	private native int _getPortalPos()
+	private static native int _getPortalPos(long pointer)
 	throws SQLException;
 
-	private native TupleDesc _getTupleDesc()
+	private static native TupleDesc _getTupleDesc(long pointer)
 	throws SQLException;
 
-	private native int _fetch(boolean forward, int count)
+	private static native int _fetch(long pointer, boolean forward, int count)
 	throws SQLException;
 
-	private native void _invalidate();
+	private static native void _close(long pointer);
 
-	private native boolean _isAtEnd()
+	private static native boolean _isAtEnd(long pointer)
 	throws SQLException;
 
-	private native boolean _isAtStart()
+	private static native boolean _isAtStart(long pointer)
 	throws SQLException;
 	
-	private native boolean _isPosOverflow()
+	private static native boolean _isPosOverflow(long pointer)
 	throws SQLException;
 
-	private native int _move(boolean forward, int count)
+	private static native int _move(long pointer, boolean forward, int count)
 	throws SQLException;
 }
