@@ -9,9 +9,12 @@ package org.postgresql.pljava.internal;
 import java.sql.Connection;
 import java.sql.SQLException;
 import java.sql.Statement;
-import java.util.ArrayList;
 import java.util.HashMap;
 
+import org.postgresql.pljava.ObjectPool;
+import org.postgresql.pljava.SavepointListener;
+import org.postgresql.pljava.TransactionListener;
+
 
 /**
  * An instance of this interface reflects the current session. The attribute
@@ -19,9 +22,8 @@ import java.util.HashMap;
  *
  * @author Thomas Hallgren
  */
-public class Session implements EOXactListener, org.postgresql.pljava.Session
+public class Session implements org.postgresql.pljava.Session
 {
-	private final ArrayList m_xactListeners = new ArrayList();
 	private final TransactionalMap m_attributes = new TransactionalMap(new HashMap());
 
 	/**
@@ -30,8 +32,16 @@ public class Session implements EOXactListener, org.postgresql.pljava.Session
 	 */
 	public void addTransactionListener(TransactionListener listener)
 	{
-		if(!m_xactListeners.contains(listener))
-			m_xactListeners.add(listener);
+		XactListener.addListener(listener);
+	}
+
+	/**
+	 * Adds the specified listener to the list of listeners that will
+	 * receive savepoint events.
+	 */
+	public void addSavepointListener(SavepointListener listener)
+	{
+		SubXactListener.addListener(listener);
 	}
 
 	public Object getAttribute(String attributeName)
@@ -39,13 +49,9 @@ public class Session implements EOXactListener, org.postgresql.pljava.Session
 		return m_attributes.get(attributeName);
 	}
 
-	/**
-	 * Returns the list of listeners that will receive transactional events.
-	 */
-	public TransactionListener[] getTransactionListeners()
+	public ObjectPool getObjectPool(Class cls)
 	{
-		return (TransactionListener[])m_xactListeners.toArray(
-				new TransactionListener[m_xactListeners.size()]);
+		return ObjectPoolImpl.getObjectPool(cls);
 	}
 
 	/**
@@ -64,34 +70,6 @@ public class Session implements EOXactListener, org.postgresql.pljava.Session
 		return AclId.getSessionUser().getName();
 	}
 
-	
-	public void onEOXact(boolean isCommit)
-	{
-		if(isCommit)
-			m_attributes.commit();
-		else
-			m_attributes.abort();
-
-		int top = m_xactListeners.size();
-		if(top == 0)
-			return;
-
-		TransactionEvent te = new TransactionEvent(this);
-
-		// Take a snapshot. Handlers might unregister during event processing
-		//
-		TransactionListener[] listeners = this.getTransactionListeners();
-		if(isCommit)
-		{
-			for(int idx = 0; idx < top; ++idx)
-				listeners[idx].afterCommit(te);
-		}
-		else
-		{
-			for(int idx = 0; idx < top; ++idx)
-				listeners[idx].afterAbort(te);
-		}	
-	}
 
 	public void removeAttribute(String attributeName)
 	{
@@ -109,7 +87,16 @@ public class Session implements EOXactListener, org.postgresql.pljava.Session
 	 */
 	public void removeTransactionListener(TransactionListener listener)
 	{
-		m_xactListeners.remove(listener);
+		XactListener.removeListener(listener);
+	}
+
+	/**
+	 * Removes the specified listener from the list of listeners that will
+	 * receive savepoint events.
+	 */
+	public void removeSavepointListener(SavepointListener listener)
+	{
+		SubXactListener.removeListener(listener);
 	}
 
 	public boolean executeAsSessionUser(Connection conn, String statement)
@@ -135,5 +122,19 @@ public class Session implements EOXactListener, org.postgresql.pljava.Session
 		}
 	}
 
+	/**
+	 * Called from native code when the JVM is instantiated.
+	 */
+	static long init()
+	throws SQLException
+	{
+		ELogHandler.init();
+		
+		// Should be replace with a Thread.getId() once we abandon
+		// Java 1.4
+		//
+		return System.identityHashCode(Thread.currentThread());
+	}
+
 	private static native void _setUser(AclId userId);
 }
