@@ -16,11 +16,18 @@ import org.postgresql.pljava.jdbc.TriggerResultSet;
  * 
  * @author Thomas Hallgren
  */
-public class TriggerData extends NativeStruct implements org.postgresql.pljava.TriggerData
+public class TriggerData extends JavaWrapper implements org.postgresql.pljava.TriggerData
 {
 	private Relation m_relation;
 	private TriggerResultSet m_old = null;
 	private TriggerResultSet m_new = null;
+	private Tuple m_newTuple;
+	private Tuple m_triggerTuple;
+
+	TriggerData(long pointer)
+	{
+		super(pointer);
+	}
 
 	/**
 	 * Returns the ResultSet that represents the new row. This ResultSet will
@@ -50,7 +57,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 				
 		// Triggers fired after will always have a read-only row
 		//
-		m_new = new TriggerResultSet(this.relation().getTupleDesc(), tuple, this.isFiredAfter());
+		m_new = new TriggerResultSet(this.getRelation().getTupleDesc(), tuple, this.isFiredAfter());
 		return m_new;
 	}
 
@@ -72,7 +79,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 
 		if (this.isFiredByInsert() || this.isFiredForStatement())
 			return null;
-		m_old = new TriggerResultSet(this.relation().getTupleDesc(), this.getTriggerTuple(), true);
+		m_old = new TriggerResultSet(this.getRelation().getTupleDesc(), this.getTriggerTuple(), true);
 		return m_old;
 	}
 
@@ -102,7 +109,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 				Tuple original = (Tuple)changes[0];
 				int[] indexes = (int[])changes[1];
 				Object[] values = (Object[])changes[2];
-				return this.relation().modifyTuple(original, indexes, values);
+				return this.getRelation().modifyTuple(original, indexes, values);
 			}
 		}
 
@@ -116,7 +123,11 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	public String getTableName()
 	throws SQLException
 	{
-		return this.relation().getName();
+		return this.getRelation().getName();
+	}
+
+	public String getSchemaName() throws SQLException {
+		return this.getRelation().getSchema();
 	}
 
 	/**
@@ -128,10 +139,14 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	public Relation getRelation()
 	throws SQLException
 	{
-		synchronized(Backend.THREADLOCK)
+		if(m_relation == null)
 		{
-			return this._getRelation();
+			synchronized(Backend.THREADLOCK)
+			{
+				m_relation = _getRelation(this.getNativePointer());
+			}
 		}
+		return m_relation;
 	}
 
 	/**
@@ -150,10 +165,14 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	public Tuple getTriggerTuple()
 	throws SQLException
 	{
-		synchronized(Backend.THREADLOCK)
+		if(m_triggerTuple == null)
 		{
-			return this._getTriggerTuple();
+			synchronized(Backend.THREADLOCK)
+			{
+				m_triggerTuple = _getTriggerTuple(this.getNativePointer());
+			}
 		}
+		return m_triggerTuple;
 	}
 
 	/**
@@ -170,10 +189,14 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	public Tuple getNewTuple()
 	throws SQLException
 	{
-		synchronized(Backend.THREADLOCK)
+		if(m_newTuple == null)
 		{
-			return this._getNewTuple();
+			synchronized(Backend.THREADLOCK)
+			{
+				m_newTuple = _getNewTuple(this.getNativePointer());
+			}
 		}
+		return m_newTuple;
 	}
 
 	/**
@@ -189,7 +212,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._getArguments();
+			return _getArguments(this.getNativePointer());
 		}
 	}
 
@@ -205,7 +228,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._getName();
+			return _getName(this.getNativePointer());
 		}
 	}
 
@@ -221,7 +244,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredAfter();
+			return _isFiredAfter(this.getNativePointer());
 		}
 	}
 
@@ -237,7 +260,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredBefore();
+			return _isFiredBefore(this.getNativePointer());
 		}
 	}
 
@@ -253,7 +276,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredForEachRow();
+			return _isFiredForEachRow(this.getNativePointer());
 		}
 	}
 
@@ -269,7 +292,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredForStatement();
+			return _isFiredForStatement(this.getNativePointer());
 		}
 	}
 
@@ -284,7 +307,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredByDelete();
+			return _isFiredByDelete(this.getNativePointer());
 		}
 	}
 
@@ -299,7 +322,7 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredByInsert();
+			return _isFiredByInsert(this.getNativePointer());
 		}
 	}
 
@@ -314,28 +337,21 @@ public class TriggerData extends NativeStruct implements org.postgresql.pljava.T
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			return this._isFiredByUpdate();
+			return _isFiredByUpdate(this.getNativePointer());
 		}
 	}
 
-	private final Relation relation()
-	throws SQLException
-	{
-		if(m_relation == null)
-			m_relation = this.getRelation();
-		return m_relation;
-	}
-
-	private native Relation _getRelation() throws SQLException;
-	private native Tuple _getTriggerTuple() throws SQLException;
-	private native Tuple _getNewTuple() throws SQLException;
-	private native String[] _getArguments() throws SQLException;
-	private native String _getName() throws SQLException;
-	private native boolean _isFiredAfter() throws SQLException;
-	private native boolean _isFiredBefore() throws SQLException;
-	private native boolean _isFiredForEachRow() throws SQLException;
-	private native boolean _isFiredForStatement() throws SQLException;
-	private native boolean _isFiredByDelete() throws SQLException;
-	private native boolean _isFiredByInsert() throws SQLException;
-	private native boolean _isFiredByUpdate() throws SQLException;
+	protected native void _free(long pointer);
+	private static native Relation _getRelation(long pointer) throws SQLException;
+	private static native Tuple _getTriggerTuple(long pointer) throws SQLException;
+	private static native Tuple _getNewTuple(long pointer) throws SQLException;
+	private static native String[] _getArguments(long pointer) throws SQLException;
+	private static native String _getName(long pointer) throws SQLException;
+	private static native boolean _isFiredAfter(long pointer) throws SQLException;
+	private static native boolean _isFiredBefore(long pointer) throws SQLException;
+	private static native boolean _isFiredForEachRow(long pointer) throws SQLException;
+	private static native boolean _isFiredForStatement(long pointer) throws SQLException;
+	private static native boolean _isFiredByDelete(long pointer) throws SQLException;
+	private static native boolean _isFiredByInsert(long pointer) throws SQLException;
+	private static native boolean _isFiredByUpdate(long pointer) throws SQLException;
 }
