@@ -10,6 +10,7 @@ import java.io.ByteArrayInputStream;
 import java.io.InputStream;
 import java.io.IOException;
 import java.io.OutputStream;
+import java.nio.ByteBuffer;
 import java.sql.Blob;
 import java.sql.SQLException;
 
@@ -171,6 +172,37 @@ public class BlobValue extends InputStream implements Blob
 		return rs;
 	}
 
+	/**
+	 * Called from within...
+	 * @param buf a buffer that reflects the internally allocated bytea buffer.
+	 * This size of this buffer will be exactly the size returned by a call to
+	 * {@link #length()}.
+	 * @throws IOException
+	 */
+	public void getContents(ByteBuffer buf)
+	throws IOException
+	{
+		int rs = 0;
+		if(buf.hasArray())
+		{
+			byte[] bytes = buf.array();
+			rs = m_stream.read(bytes);
+		}
+		else
+		{
+			byte[] trBuf = new byte[1024];
+			int br;
+			while((br = m_stream.read(trBuf)) > 0)
+			{
+				buf.put(trBuf, 0, br);
+				rs += br;
+			}
+		}
+		if(rs != m_nBytes)
+			throw new IOException("Not all bytes could be read");
+		m_streamPos += rs;
+	}
+
 	public synchronized int read(byte[] b, int off,  int len)
 	throws IOException
 	{
