@@ -12,6 +12,7 @@ import java.util.ArrayList;
 import java.util.logging.Logger;
 
 import org.postgresql.pljava.internal.Backend;
+import org.postgresql.pljava.internal.PgSavepoint;
 
 /**
  * @author Thomas Hallgren
@@ -31,7 +32,7 @@ public class Invocation
 	/**
 	 * Top level savepoint relative to this invocation.
 	 */
-	private SPISavepoint m_savepoint;
+	private PgSavepoint m_savepoint;
 
 	private Invocation(int level)
 	{
@@ -49,7 +50,7 @@ public class Invocation
 	/**
 	 * @return Returns the savePoint.
 	 */
-	final SPISavepoint getSavepoint()
+	final PgSavepoint getSavepoint()
 	{
 		return m_savepoint;
 	}
@@ -80,7 +81,7 @@ public class Invocation
 	/**
 	 * @param savepoint The savepoint to set.
 	 */
-	final void setSavepoint(SPISavepoint savepoint)
+	final void setSavepoint(PgSavepoint savepoint)
 	{
 		m_savepoint = savepoint;
 	}
@@ -95,7 +96,7 @@ public class Invocation
 		try
 		{
 			if(m_savepoint != null)
-				m_savepoint.onInvocationExit();
+				m_savepoint.onInvocationExit(SPIDriver.getDefault());
 
 			if(m_preparedStatements != null)
 			{
@@ -128,14 +129,20 @@ public class Invocation
 	{
 		synchronized(Backend.THREADLOCK)
 		{
-			Invocation curr;
+			Invocation curr = _getCurrent();
+			if(curr != null)
+				return curr;
+
 			int level = _getNestingLevel();
 			int top = s_levels.length;
-			if(level <= top)
+			if(level < top)
 			{
 				curr = s_levels[level];
 				if(curr != null)
+				{
+					curr._register();
 					return curr;
+				}
 			}
 			else
 			{
@@ -165,6 +172,12 @@ public class Invocation
 	 */
 	private native void  _register();
 	
+	/**
+	 * Returns the current invocation or null if no invocation has been
+	 * registered yet.
+	 */
+	private native static Invocation  _getCurrent();
+
 	/**
 	 * Returns the current nesting level
 	 */
