@@ -6,6 +6,7 @@
  */
 package org.postgresql.pljava.jdbc;
 
+import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 import java.util.ArrayList;
 
@@ -52,6 +53,12 @@ public class TriggerResultSet extends SingleRowResultSet
 		return m_readOnly ? CONCUR_READ_ONLY : CONCUR_UPDATABLE;
 	}
 
+	public ResultSetMetaData getMetaData()
+	throws SQLException
+	{
+		return new SPIResultSetMetaData(m_tupleDesc);
+	}
+
 	/**
 	 * Returns <code>true</code> if this row has been updated.
 	 */
